@@ -82,6 +82,10 @@ var configByHost = map[string]oauth2.Config{
 		Endpoint: replaceHost(endpoints.GitLab, "gitlab.alpinelinux.org"),
 		Scopes:   []string{"read_repository", "write_repository"}},
 	// https://gitea.com/user/settings/applications/oauth2/218
+	"invent.kde.org": {
+		ClientID: "36ee2eeb4f179b38db8f066fd4ca9751f4b003653a205fc1422f110ca60181f6",
+		Endpoint: replaceHost(endpoints.GitLab, "invent.kde.org"),
+		Scopes:   []string{"read_repository", "write_repository"}},
 	"gitea.com": {
 		ClientID: "e13f8ebc-398d-4091-9481-5a37a76b51f6",
 		Endpoint: oauth2.Endpoint{AuthURL: "https://gitea.com/login/oauth/authorize", TokenURL: "https://gitea.com/login/oauth/access_token"}},
@@ -135,6 +139,7 @@ func parse(input string) map[string]string {
 }
 
 func main() {
+	ctx := context.Background()
 	flag.BoolVar(&verbose, "verbose", false, "log debug information to stderr")
 	var device bool
 	flag.BoolVar(&device, "device", false, "instead of opening a web browser locally, print a code to enter on another device")
@@ -176,7 +181,7 @@ func main() {
 			fmt.Fprintln(os.Stderr, "input:", pairs)
 		}
 		host := pairs["host"]
-		looksLikeGitLab := strings.HasPrefix(host, "gitlab.") || strings.Contains(pairs["wwwauth[]"], `Realm="GitLab"`)
+		looksLikeGitLab := strings.HasPrefix(host, "gitlab.") || strings.Contains(pairs["wwwauth[]"], `realm="GitLab"`)
 		looksLikeGitea := strings.Contains(pairs["wwwauth[]"], `realm="Gitea"`)
 		looksLikeGitHub := strings.HasPrefix(host, "github.") || strings.Contains(pairs["wwwauth[]"], `realm="GitHub"`)
 		urll := fmt.Sprintf("%s://%s", pairs["protocol"], host)
@@ -250,6 +255,9 @@ func main() {
 			if looksLikeGitLab {
 				fmt.Fprintf(os.Stderr, "It looks like you're authenticating to a GitLab instance! To configure git-credential-oauth for host %s, follow the instructions at https://github.com/hickford/git-credential-oauth/issues/18. You may need to register an OAuth application at https://%s/-/profile/applications\n", host, host)
 			}
+			if looksLikeGitHub {
+				fmt.Fprintf(os.Stderr, "It looks like you're authenticating to GitHub Enterprise Server. See issue https://github.com/hickford/git-credential-oauth/issues/39 and workaround https://github.com/hickford/git-credential-oauth/issues/39#issuecomment-1747514543.\n")
+			}
 			return
 		}
 
@@ -259,7 +267,7 @@ func main() {
 			if verbose {
 				fmt.Fprintln(os.Stderr, "refreshing token...")
 			}
-			token, err = c.TokenSource(context.Background(), &oauth2.Token{RefreshToken: pairs["oauth_refresh_token"]}).Token()
+			token, err = c.TokenSource(ctx, &oauth2.Token{RefreshToken: pairs["oauth_refresh_token"]}).Token()
 			if err != nil {
 				fmt.Fprintln(os.Stderr, "error during OAuth token refresh", err)
 			}
@@ -268,9 +276,9 @@ func main() {
 		if token == nil {
 			// Generate new token (opens browser, may require user input)
 			if device {
-				token, err = getDeviceToken(c)
+				token, err = getDeviceToken(ctx, c)
 			} else {
-				token, err = getToken(c)
+				token, err = getToken(ctx, c)
 			}
 			if err != nil {
 				log.Fatalln(err)
@@ -356,7 +364,7 @@ var template string = `<!DOCTYPE html>
 </body>
 </html>`
 
-func getToken(c oauth2.Config) (*oauth2.Token, error) {
+func getToken(ctx context.Context, c oauth2.Config) (*oauth2.Token, error) {
 	state := oauth2.GenerateVerifier()
 	queries := make(chan url.Values)
 	handler := http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
@@ -425,19 +433,19 @@ func getToken(c oauth2.Config) (*oauth2.Token, error) {
 		return nil, fmt.Errorf("state mismatch")
 	}
 	code := query.Get("code")
-	return c.Exchange(context.Background(), code, oauth2.VerifierOption(verifier))
+	return c.Exchange(ctx, code, oauth2.VerifierOption(verifier))
 }
 
-func getDeviceToken(c oauth2.Config) (*oauth2.Token, error) {
-	deviceAuth, err := c.DeviceAuth(context.Background())
+func getDeviceToken(ctx context.Context, c oauth2.Config) (*oauth2.Token, error) {
+	deviceAuth, err := c.DeviceAuth(ctx)
 	if err != nil {
 		log.Fatalln(err)
 	}
 	if verbose {
-		fmt.Fprintln(os.Stderr, deviceAuth)
+		fmt.Fprintf(os.Stderr, "%+v\n", deviceAuth)
 	}
 	fmt.Fprintf(os.Stderr, "Please enter code %s at %s\n", deviceAuth.UserCode, deviceAuth.VerificationURI)
-	return c.DeviceAccessToken(context.Background(), deviceAuth)
+	return c.DeviceAccessToken(ctx, deviceAuth)
 }
 
 func replaceHost(e oauth2.Endpoint, host string) oauth2.Endpoint {
